@@ -114,7 +114,7 @@ class AuthenticatedHandler(web.RequestHandler):
         name = escape.native_str(name)
         expires = datetime.datetime.utcnow() - datetime.timedelta(days=365)
 
-        morsel = Morsel()
+        morsel: Morsel = Morsel()
         morsel.set(name, "", '""')
         morsel["expires"] = httputil.format_timestamp(expires)
         morsel["path"] = path
@@ -193,7 +193,23 @@ class AuthenticatedHandler(web.RequestHandler):
 
     @property
     def authorizer(self):
-        return self.settings.get("authorizer")
+        if "authorizer" not in self.settings:
+            warnings.warn(
+                "The Tornado web application does not have an 'authorizer' defined "
+                "in its settings. In future releases of jupyter_server, this will "
+                "be a required key for all subclasses of `JupyterHandler`. For an "
+                "example, see the jupyter_server source code for how to "
+                "add an authorizer to the tornado settings: "
+                "https://github.com/jupyter-server/jupyter_server/blob/"
+                "653740cbad7ce0c8a8752ce83e4d3c2c754b13cb/jupyter_server/serverapp.py"
+                "#L234-L256",
+            )
+            from jupyter_server.auth import AllowAllAuthorizer
+
+            self.settings["authorizer"] = AllowAllAuthorizer(
+                config=self.settings.get("config", None)
+            )
+        return self.settings["authorizer"]
 
 
 class JupyterHandler(AuthenticatedHandler):
@@ -241,7 +257,7 @@ class JupyterHandler(AuthenticatedHandler):
         return self.settings.get("mathjax_config", "TeX-AMS-MML_HTMLorMML-full,Safe")
 
     @property
-    def base_url(self):
+    def base_url(self) -> str:
         return self.settings.get("base_url", "/")
 
     @property
@@ -476,7 +492,9 @@ class JupyterHandler(AuthenticatedHandler):
             return True
 
         # Remove port (e.g. ':8888') from host
-        host = re.match(r"^(.*?)(:\d+)?$", self.request.host).group(1)
+        match = re.match(r"^(.*?)(:\d+)?$", self.request.host)
+        assert match is not None
+        host = match.group(1)
 
         # Browsers format IPv6 addresses like [::1]; we need to remove the []
         if host.startswith("[") and host.endswith("]"):
@@ -567,7 +585,7 @@ class JupyterHandler(AuthenticatedHandler):
         exc_info = kwargs.get("exc_info")
         message = ""
         status_message = responses.get(status_code, "Unknown HTTP Error")
-        exception = "(unknown)"
+
         if exc_info:
             exception = exc_info[1]
             # get the custom message, if defined
@@ -580,6 +598,8 @@ class JupyterHandler(AuthenticatedHandler):
             reason = getattr(exception, "reason", "")
             if reason:
                 status_message = reason
+        else:
+            exception = "(unknown)"
 
         # build template namespace
         ns = dict(
@@ -602,6 +622,8 @@ class JupyterHandler(AuthenticatedHandler):
 class APIHandler(JupyterHandler):
     """Base class for API handlers"""
 
+    _user_cache: dict
+
     def prepare(self):
         if not self.check_origin():
             raise web.HTTPError(404)
@@ -611,7 +633,7 @@ class APIHandler(JupyterHandler):
         """APIHandler errors are JSON, not human pages"""
         self.set_header("Content-Type", "application/json")
         message = responses.get(status_code, "Unknown HTTP Error")
-        reply = {
+        reply: dict = {
             "message": message,
         }
         exc_info = kwargs.get("exc_info")
@@ -733,13 +755,14 @@ class AuthenticatedFileHandler(JupyterHandler, web.StaticFileHandler):
 
     @web.authenticated
     def get(self, path):
-        if os.path.splitext(path)[1] == ".ipynb" or self.get_argument("download", False):
+        if os.path.splitext(path)[1] == ".ipynb" or self.get_argument("download", None):
             name = path.rsplit("/", 1)[-1]
             self.set_attachment_header(name)
 
         return web.StaticFileHandler.get(self, path)
 
     def get_content_type(self):
+        assert self.absolute_path is not None
         path = self.absolute_path.strip("/")
         if "/" in path:
             _, name = path.rsplit("/", 1)
@@ -818,7 +841,8 @@ class FileFindHandler(JupyterHandler, web.StaticFileHandler):
     """subclass of StaticFileHandler for serving files from a search path"""
 
     # cache search results, don't search for files more than once
-    _static_paths = {}
+    _static_paths: dict = {}
+    root: tuple
 
     def set_headers(self):
         super().set_headers()
@@ -882,6 +906,7 @@ class TrailingSlashHandler(web.RequestHandler):
     """
 
     def get(self):
+        assert self.request.uri is not None
         path, *rest = self.request.uri.partition("?")
         # trim trailing *and* leading /
         # to avoid misinterpreting repeated '//'
