@@ -1,12 +1,14 @@
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
+import asyncio
 import datetime
 import json
 import os
 from logging import Logger
-from queue import Queue
+from queue import Empty, Queue
 from threading import Thread
-from typing import Dict
+from time import monotonic
+from typing import Any, Dict, Optional
 
 import websocket
 from jupyter_client.asynchronous.client import AsyncKernelClient
@@ -92,7 +94,7 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
             The uuid of the kernel.
         """
         model = None
-        km = self.get_kernel(kernel_id)
+        km = self.get_kernel(str(kernel_id))
         if km:
             model = km.kernel
         return model
@@ -166,13 +168,14 @@ class GatewayMappingKernelManager(AsyncMappingKernelManager):
 
     async def shutdown_all(self, now=False):
         """Shutdown all kernels."""
-        for kernel_id in self._kernels:
+        kids = list(self._kernels)
+        for kernel_id in kids:
             km = self.get_kernel(kernel_id)
             await km.shutdown_kernel(now=now)
             self.remove_kernel(kernel_id)
 
     async def cull_kernels(self):
-        """Override cull_kernels so we can be sure their state is current."""
+        """Override cull_kernels, so we can be sure their state is current."""
         await self.list_kernels()
         await super().cull_kernels()
 
@@ -295,7 +298,7 @@ class GatewaySessionManager(SessionManager):
     kernel_manager = Instance("jupyter_server.gateway.managers.GatewayMappingKernelManager")
 
     async def kernel_culled(self, kernel_id):
-        """Checks if the kernel is still considered alive and returns true if its not found."""
+        """Checks if the kernel is still considered alive and returns true if it's not found."""
         kernel = None
         try:
             km = self.kernel_manager.get_kernel(kernel_id)
@@ -387,7 +390,7 @@ class GatewayKernelManager(AsyncKernelManager):
             if isinstance(self.parent, AsyncMappingKernelManager):
                 # Update connections only if there's a mapping kernel manager parent for
                 # this kernel manager.  The current kernel manager instance may not have
-                # an parent instance if, say, a server extension is using another application
+                # a parent instance if, say, a server extension is using another application
                 # (e.g., papermill) that uses a KernelManager instance directly.
                 self.parent._kernel_connections[self.kernel_id] = int(model["connections"])
 
@@ -448,12 +451,19 @@ class GatewayKernelManager(AsyncKernelManager):
 
         if self.has_kernel:
             self.log.debug("Request shutdown kernel at: %s", self.kernel_url)
-            response = await gateway_request(self.kernel_url, method="DELETE")
-            self.log.debug("Shutdown kernel response: %d %s", response.code, response.reason)
+            try:
+                response = await gateway_request(self.kernel_url, method="DELETE")
+                self.log.debug("Shutdown kernel response: %d %s", response.code, response.reason)
+            except web.HTTPError as error:
+                if error.status_code == 404:
+                    self.log.debug("Shutdown kernel response: kernel not found (ignored)")
+                else:
+                    raise
 
     async def restart_kernel(self, **kw):
         """Restarts a kernel via HTTP."""
         if self.has_kernel:
+            assert self.kernel_url is not None
             kernel_url = self.kernel_url + "/restart"
             self.log.debug("Request restart kernel at: %s", kernel_url)
             response = await gateway_request(kernel_url, method="POST", body=json_encode({}))
@@ -462,6 +472,7 @@ class GatewayKernelManager(AsyncKernelManager):
     async def interrupt_kernel(self):
         """Interrupts the kernel via an HTTP request."""
         if self.has_kernel:
+            assert self.kernel_url is not None
             kernel_url = self.kernel_url + "/interrupt"
             self.log.debug("Request interrupt kernel at: %s", kernel_url)
             response = await gateway_request(kernel_url, method="POST", body=json_encode({}))
@@ -486,17 +497,36 @@ KernelManagerABC.register(GatewayKernelManager)
 
 class ChannelQueue(Queue):
 
-    channel_name: str = None
+    channel_name: Optional[str] = None
+    response_router_finished: bool
 
     def __init__(self, channel_name: str, channel_socket: websocket.WebSocket, log: Logger):
         super().__init__()
         self.channel_name = channel_name
         self.channel_socket = channel_socket
         self.log = log
+        self.response_router_finished = False
 
-    async def get_msg(self, *args, **kwargs) -> dict:
+    async def _async_get(self, timeout=None):
+        if timeout is None:
+            timeout = float("inf")
+        elif timeout < 0:
+            raise ValueError("'timeout' must be a non-negative number")
+        end_time = monotonic() + timeout
+
+        while True:
+            try:
+                return self.get(block=False)
+            except Empty:
+                if self.response_router_finished:
+                    raise RuntimeError("Response router had finished")
+                if monotonic() > end_time:
+                    raise
+                await asyncio.sleep(0)
+
+    async def get_msg(self, *args: Any, **kwargs: Any) -> dict:
         timeout = kwargs.get("timeout", 1)
-        msg = self.get(timeout=timeout)
+        msg = await self._async_get(timeout=timeout)
         self.log.debug(
             "Received message on channel: {}, msg_id: {}, msg_type: {}".format(
                 self.channel_name, msg["msg_id"], msg["msg_type"] if msg else "null"
@@ -516,7 +546,7 @@ class ChannelQueue(Queue):
 
     @staticmethod
     def serialize_datetime(dt):
-        if isinstance(dt, (datetime.date, datetime.datetime)):
+        if isinstance(dt, datetime.datetime):
             return dt.timestamp()
         return None
 
@@ -571,14 +601,21 @@ class GatewayKernelClient(AsyncKernelClient):
 
     # flag for whether execute requests should be allowed to call raw_input:
     allow_stdin = False
-    _channels_stopped = False
-    _channel_queues = {}
+    _channels_stopped: bool
+    _channel_queues: Optional[Dict[str, ChannelQueue]]
+    _control_channel: Optional[ChannelQueue]
+    _hb_channel: Optional[ChannelQueue]
+    _stdin_channel: Optional[ChannelQueue]
+    _iopub_channel: Optional[ChannelQueue]
+    _shell_channel: Optional[ChannelQueue]
 
-    def __init__(self, **kwargs):
+    def __init__(self, kernel_id, **kwargs):
         super().__init__(**kwargs)
-        self.kernel_id = kwargs["kernel_id"]
-        self.channel_socket = None
-        self.response_router = None
+        self.kernel_id = kernel_id
+        self.channel_socket: Optional[websocket.WebSocket] = None
+        self.response_router: Optional[Thread] = None
+        self._channels_stopped = False
+        self._channel_queues = {}
 
     # --------------------------------------------------------------------------
     # Channel management methods
@@ -588,7 +625,7 @@ class GatewayKernelClient(AsyncKernelClient):
         """Starts the channels for this kernel.
 
         For this class, we establish a websocket connection to the destination
-        and setup the channel-based queues on which applicable messages will
+        and set up the channel-based queues on which applicable messages will
         be posted.
         """
 
@@ -599,10 +636,11 @@ class GatewayKernelClient(AsyncKernelClient):
             "channels",
         )
         # Gather cert info in case where ssl is desired...
-        ssl_options = {}
-        ssl_options["ca_certs"] = GatewayClient.instance().ca_certs
-        ssl_options["certfile"] = GatewayClient.instance().client_cert
-        ssl_options["keyfile"] = GatewayClient.instance().client_key
+        ssl_options = {
+            "ca_certs": GatewayClient.instance().ca_certs,
+            "certfile": GatewayClient.instance().client_cert,
+            "keyfile": GatewayClient.instance().client_key,
+        }
 
         self.channel_socket = websocket.create_connection(
             ws_url,
@@ -610,13 +648,14 @@ class GatewayKernelClient(AsyncKernelClient):
             enable_multithread=True,
             sslopt=ssl_options,
         )
-        self.response_router = Thread(target=self._route_responses)
-        self.response_router.start()
 
         await ensure_async(
             super().start_channels(shell=shell, iopub=iopub, stdin=stdin, hb=hb, control=control)
         )
 
+        self.response_router = Thread(target=self._route_responses)
+        self.response_router.start()
+
     def stop_channels(self):
         """Stops all the running channels for this kernel.
 
@@ -627,7 +666,9 @@ class GatewayKernelClient(AsyncKernelClient):
         self._channels_stopped = True
         self.log.debug("Closing websocket connection")
 
+        assert self.channel_socket is not None
         self.channel_socket.close()
+        assert self.response_router is not None
         self.response_router.join()
 
         if self._channel_queues:
@@ -641,7 +682,9 @@ class GatewayKernelClient(AsyncKernelClient):
         """Get the shell channel object for this kernel."""
         if self._shell_channel is None:
             self.log.debug("creating shell channel queue")
+            assert self.channel_socket is not None
             self._shell_channel = ChannelQueue("shell", self.channel_socket, self.log)
+            assert self._channel_queues is not None
             self._channel_queues["shell"] = self._shell_channel
         return self._shell_channel
 
@@ -650,7 +693,9 @@ class GatewayKernelClient(AsyncKernelClient):
         """Get the iopub channel object for this kernel."""
         if self._iopub_channel is None:
             self.log.debug("creating iopub channel queue")
+            assert self.channel_socket is not None
             self._iopub_channel = ChannelQueue("iopub", self.channel_socket, self.log)
+            assert self._channel_queues is not None
             self._channel_queues["iopub"] = self._iopub_channel
         return self._iopub_channel
 
@@ -659,7 +704,9 @@ class GatewayKernelClient(AsyncKernelClient):
         """Get the stdin channel object for this kernel."""
         if self._stdin_channel is None:
             self.log.debug("creating stdin channel queue")
+            assert self.channel_socket is not None
             self._stdin_channel = ChannelQueue("stdin", self.channel_socket, self.log)
+            assert self._channel_queues is not None
             self._channel_queues["stdin"] = self._stdin_channel
         return self._stdin_channel
 
@@ -668,7 +715,9 @@ class GatewayKernelClient(AsyncKernelClient):
         """Get the hb channel object for this kernel."""
         if self._hb_channel is None:
             self.log.debug("creating hb channel queue")
+            assert self.channel_socket is not None
             self._hb_channel = HBChannelQueue("hb", self.channel_socket, self.log)
+            assert self._channel_queues is not None
             self._channel_queues["hb"] = self._hb_channel
         return self._hb_channel
 
@@ -677,7 +726,9 @@ class GatewayKernelClient(AsyncKernelClient):
         """Get the control channel object for this kernel."""
         if self._control_channel is None:
             self.log.debug("creating control channel queue")
+            assert self.channel_socket is not None
             self._control_channel = ChannelQueue("control", self.channel_socket, self.log)
+            assert self._channel_queues is not None
             self._channel_queues["control"] = self._control_channel
         return self._control_channel
 
@@ -691,20 +742,27 @@ class GatewayKernelClient(AsyncKernelClient):
         """
         try:
             while not self._channels_stopped:
+                assert self.channel_socket is not None
                 raw_message = self.channel_socket.recv()
                 if not raw_message:
                     break
                 response_message = json_decode(utf8(raw_message))
                 channel = response_message["channel"]
+                assert self._channel_queues is not None
                 self._channel_queues[channel].put_nowait(response_message)
 
         except websocket.WebSocketConnectionClosedException:
-            pass  # websocket closure most likely due to shutdown
+            pass  # websocket closure most likely due to shut down
 
         except BaseException as be:
             if not self._channels_stopped:
                 self.log.warning(f"Unexpected exception encountered ({be})")
 
+        # Notify channel queues that this thread had finished and no more messages are being received
+        assert self._channel_queues is not None
+        for channel_queue in self._channel_queues.values():
+            channel_queue.response_router_finished = True
+
         self.log.debug("Response router thread exiting...")
 
 
